@@ -63,8 +63,16 @@ namespace YaHTTP {
             while(k2 < static_cast<int>(req->url.path.size()) && req->url.path[k2] != url[k1+1]) k2++;
             pos2 = k2;
             params[pname] = funcptr::tie(pos1,pos2);
+            if (k2 > 0) {
+              k2--;
+            }
+            else {
+              // If k2 is zero, do not decrement it and then increment at bottom of loop
+              // Only increment k1 and continue loop
+              k1++;
+              continue;
+            }
           }
-          k2--;
         }
         else if (url[k1] != req->url.path[k2]) {
           break;
@@ -98,7 +106,7 @@ namespace YaHTTP {
 
   void Router::printRoutes(std::ostream &os) {
     for(TRouteList::iterator i = routes.begin(); i != routes.end(); i++) {
-#ifdef HAVE_CXX11
+#if __cplusplus >= 201103L
       std::streamsize ss = os.width();
       std::ios::fmtflags ff = os.setf(std::ios::left);
       os.width(10);
@@ -122,7 +130,7 @@ namespace YaHTTP {
 
     bool found = false;
     for(TRouteList::iterator i = routes.begin(); !found && i != routes.end(); i++) {
-#ifdef HAVE_CXX11
+#if __cplusplus >= 201103L
       if (std::get<3>(*i) == name) { mask = std::get<1>(*i); method = std::get<0>(*i); found = true; }
 #else
       if (i->get<3>() == name) { mask = i->get<1>(); method = i->get<0>(); found = true; }
