@@ -39,7 +39,6 @@ void doCarbonDump(void*)
       config.instance_name = "recursor";
     }
 
-    registerAllStats();
     PacketBuffer msg;
     for (const auto& carbonServer : config.servers) {
       ComboAddress remote(carbonServer, 2003);
@@ -50,7 +49,7 @@ void doCarbonDump(void*)
       {
         g_networkTimeoutMsec / 1000, static_cast<suseconds_t>(g_networkTimeoutMsec) % 1000 * 1000
       };
-      auto handler = std::make_shared<TCPIOHandler>("", false, s.releaseHandle(), timeout, tlsCtx, time(nullptr));
+      auto handler = std::make_shared<TCPIOHandler>("", false, s.releaseHandle(), timeout, tlsCtx);
       handler->tryConnect(SyncRes::s_tcp_fast_open_connect, remote); // we do the connect so the first attempt happens while we gather stats
 
       if (msg.empty()) {
